@@ -147,7 +147,7 @@ typedef struct
 	TDS_INT      date;	/**< date, 0 = 1900-01-01 */
 	TDS_SMALLINT offset;	/**< time offset */
 	TDS_USMALLINT time_prec:3;
-	TDS_USMALLINT _res:10;
+	TDS_USMALLINT _tds_reserved:10;
 	TDS_USMALLINT has_time:1;
 	TDS_USMALLINT has_date:1;
 	TDS_USMALLINT has_offset:1;
@@ -519,6 +519,8 @@ bool is_tds_type_valid(int type)
 #define TDS_STR_DBFILENAME	"database filename"
 /* Application Intent MSSQL 2012 support */
 #define TDS_STR_READONLY_INTENT "read-only intent"
+/* configurable cipher suite to send to openssl's SSL_set_cipher_list() function */
+#define TLS_STR_OPENSSL_CIPHERS "openssl ciphers"
 
 
 /* TODO do a better check for alignment than this */
@@ -560,6 +562,7 @@ typedef struct tds_login
 	DSTR db_filename;		/**< database filename to attach (MSSQL) */
 	DSTR cafile;			/**< certificate authorities file */
 	DSTR crlfile;			/**< certificate revocation file */
+	DSTR openssl_ciphers;
 	DSTR app_name;
 	DSTR user_name;	    	/**< account for login */
 	DSTR password;	    	/**< password of account login */
@@ -582,7 +585,7 @@ typedef struct tds_login
 
 	unsigned char option_flag2;
 
-	unsigned int bulk_copy:1;
+	unsigned int bulk_copy:1;	/**< if bulk copy should be enabled */
 	unsigned int suppress_language:1;
 	unsigned int emul_little_endian:1;
 	unsigned int gssapi_use_delegation:1;
@@ -1129,6 +1132,7 @@ struct tds_connection
 	unsigned int use_iconv:1;
 	unsigned int tds71rev1:1;
 	unsigned int pending_close:1;	/**< true is connection has pending closing (cursors or dynamic) */
+	unsigned int encrypt_single_packet:1;
 #if ENABLE_ODBC_MARS
 	unsigned int mars:1;
 
@@ -1527,9 +1531,7 @@ int tds_connection_write(TDSSOCKET *tds, const unsigned char *buf, int buflen, i
 #define TDSSELREAD  POLLIN
 #define TDSSELWRITE POLLOUT
 int tds_select(TDSSOCKET * tds, unsigned tds_sel, int timeout_seconds);
-#if ENABLE_ODBC_MARS
 void tds_connection_close(TDSCONNECTION *conn);
-#endif
 int tds_goodread(TDSSOCKET * tds, unsigned char *buf, int buflen);
 int tds_goodwrite(TDSSOCKET * tds, const unsigned char *buffer, size_t buflen);
 void tds_socket_flush(TDS_SYS_SOCKET sock);
@@ -1551,11 +1553,6 @@ int tds_append_cancel(TDSSOCKET *tds);
 TDSRET tds_append_fin(TDSSOCKET *tds);
 #else
 int tds_put_cancel(TDSSOCKET * tds);
-static inline
-void tds_connection_close(TDSCONNECTION *connection)
-{
-	tds_close_socket((TDSSOCKET* ) connection);
-}
 #endif
 
 
