@@ -16,9 +16,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -31,6 +29,19 @@
 #include <readline/readline.h>
 #include <readline/history.h>
 #endif
+
+#if HAVE_LOCALE_H
+#include <locale.h>
+#endif
+
+#if HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif /* HAVE_SYS_TYPES_H */
+
+#if HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif /* HAVE_SYS_STAT_H */
+
 #include <sybfront.h>
 #include <sybdb.h>
 #include "terminal.h"
@@ -175,8 +186,8 @@ get_printable_size(int type, int size)
 	case SYBMONEY4:
 		return 12;	/* FIX ME */
 	case SYBDATETIME:
-		return 26;	/* FIX ME */
 	case SYBDATETIME4:
+	case SYBDATETIMN:
 		return 26;	/* FIX ME */
 #if 0				/* seems not to be exported to sybdb.h */
 	case SYBBITN:
@@ -214,7 +225,6 @@ main(int argc, char *argv[])
 	int no_prompt = 0;
 	int use_encryption = 0;
 	int chained_transactions = 0;
-	const char *display_charset = "";
 	const char *cmdend = "go";
 	int headers = 0;
 	char *columnwidth = NULL;
@@ -264,9 +274,16 @@ main(int argc, char *argv[])
 	char adbuf[512];
 	DBINT convlen;
 	int printedcompute = 0;
-	BYTE *bylist;
-	int nby;
 	char adash;
+	const char *database_name = NULL;
+	int default_exit = EXIT_SUCCESS;
+
+	setlocale(LC_ALL, "");
+
+#ifdef __VMS
+	/* Convert VMS-style arguments to Unix-style */
+	parse_vms_args(&argc, &argv);
+#endif
 
 	editor = getenv("EDITOR");
 	if (!editor) {
@@ -278,7 +295,7 @@ main(int argc, char *argv[])
 
 	opterr = 0;
 	optarg = NULL;
-	while (!errflg && (c = getopt(argc, argv, "eFgpnvXYa:c:E:h:H:i:I:J:l:m:o:P:s:S:t:U:w:y:z:A:"))
+	while (!errflg && (c = getopt(argc, argv, "eFgpnvXYa:c:D:E:h:H:i:I:J:l:m:o:P:s:S:t:U:w:y:z:A:"))
 	       != -1) {
 		switch (c) {
 		case 'e':
@@ -310,10 +327,6 @@ main(int argc, char *argv[])
 		case 'Y':
 			chained_transactions = 1;
 			break;
-		case 'a':
-			display_charset = optarg;
-			errflg++;
-			break;
 		case 'c':
 			cmdend = optarg;
 			break;
@@ -377,6 +390,9 @@ main(int argc, char *argv[])
 		case 'A':
 			size = atoi(optarg);
 			break;
+		case 'D':
+			database_name = optarg;
+			break;
 		default:
 			errflg++;
 			break;
@@ -385,7 +401,7 @@ main(int argc, char *argv[])
 
 	if (errflg) {
 		fprintf(stderr, "usage: fisql [-e] [-F] [-g] [-p] [-n] [-v] [-X] [-Y]\n");
-		fprintf(stderr, "\t[-a display_charset] [-c cmdend] [-E editor]\n");
+		fprintf(stderr, "\t[-c cmdend] [-D database_name] [-E editor]\n");
 		fprintf(stderr, "\t[-h headers] [-H hostname] [-i inputfile]\n");
 		fprintf(stderr, "\t[-I interfaces_file] [-J client character set]\n");
 		fprintf(stderr, "\t[-l login_timeout] [-m errorlevel]\n");
@@ -445,6 +461,8 @@ main(int argc, char *argv[])
 		DBSETLUSER(login, username);
 	}
 	DBSETLPWD(login, password);
+	memset(password, 0, strlen(password));
+
 	if (char_set) {
 		DBSETLCHARSET(login, char_set);
 	}
@@ -489,6 +507,9 @@ main(int argc, char *argv[])
 	if (perfstats) {
 		dbsetopt(dbproc, DBSTAT, "time", 0);
 	}
+	if (database_name) {
+		dbuse(dbproc, database_name);
+	}
 
 	while (1) {
 		if (sigsetjmp(restart, 1)) {
@@ -518,23 +539,37 @@ main(int argc, char *argv[])
 			if (line == NULL) {
 				line = "exit";
 			}
-			for (cp = line; *cp && isspace((unsigned char) *cp); cp++);
+			for (cp = line; *cp && isspace((unsigned char) *cp); cp++)
+				continue;
 			if (*cp) {
 				add_history(line);
 			}
 			if (!(strncasecmp(line, "!!", 2))) {
+				int rv;
 				cp = line + 2;
-				system(cp);
-				continue;
+				switch (rv = system(cp)) {
+				case 0:
+					continue;
+				case -1:
+					fprintf(stderr,
+					    "Failed to execute `%s'\n", cp);
+					continue;
+				default:
+					fprintf(stderr, "Command `%s' exited "
+					    "with code %d\n", cp, rv);
+					continue;
+				}
 			}
 			/* XXX: isql off-by-one line count error for :r not duplicated */
 			if (!(strncasecmp(line, ":r", 2))) {
-				for (cp = line + 2; *cp && (isspace((unsigned char) *cp)); cp++);
+				for (cp = line + 2; *cp && (isspace((unsigned char) *cp)); cp++)
+					continue;
 				tfn = cp;
-				for (; *cp && !(isspace((unsigned char) *cp)); cp++);
+				for (; *cp && !(isspace((unsigned char) *cp)); cp++)
+					continue;
 				*cp = '\0';
 				if ((fp = fopen(tfn, "r")) == NULL) {
-					printf("Operating system error: Failed to open %s.\n", tfn);
+					fprintf(stderr, "Operating system error: Failed to open %s.\n", tfn);
 					continue;
 				}
 				tmpfp = rl_instream;
@@ -565,7 +600,7 @@ main(int argc, char *argv[])
 			    || (!(strcasecmp(firstword, "quit")))) {
 				reset_term();
 				dbexit();
-				exit(EXIT_SUCCESS);
+				exit(default_exit);
 			}
 			if (!(strcasecmp(firstword, "reset"))) {
 				for (i = 0; i < ibuflines; i++) {
@@ -584,9 +619,12 @@ main(int argc, char *argv[])
 			if ((!(strcasecmp(firstword, "vi")))
 			    || (!(strcasecmp(firstword, editor)))) {
 				int tmpfd;
+				mode_t old_mask;
 
 				strcpy(tmpfn, "/tmp/fisqlXXXXXX");
+				old_mask = umask(0600);
 				tmpfd = mkstemp(tmpfn);
+				umask(old_mask);
 				if ((fp = fdopen(tmpfd, "w")) == NULL) {
 					perror("fisql");
 					reset_term();
@@ -702,10 +740,6 @@ main(int argc, char *argv[])
 								}
 								collen = get_printable_column_size(dbproc, colid);
 								adash = '-';
-								bylist = dbbylist(dbproc, dbrc, &nby);
-								if (nby == 0) {
-									adash = '=';
-								}
 								for (i = 0; i < collen; i++) {
 									putchar(adash);
 								}
@@ -781,6 +815,11 @@ main(int argc, char *argv[])
 					}
 				}
 			}
+		} else {
+			/* Something failed, so change the default
+			 * exit status to reflect that.
+			 */
+			default_exit = EXIT_FAILURE;
 		}
 	}
 	reset_term();
