@@ -86,7 +86,7 @@ tds_set_passwd(TDSLOGIN * tds_login, const char *password)
 void
 tds_set_bulk(TDSLOGIN * tds_login, TDS_TINYINT enabled)
 {
-	tds_login->bulk_copy = enabled ? 0 : 1;
+	tds_login->bulk_copy = enabled ? 1 : 0;
 }
 
 bool
@@ -314,6 +314,8 @@ tds_set_spid(TDSSOCKET * tds)
 
 		switch (result_type) {
 		case TDS_ROW_RESULT:
+			if (!tds->res_info)
+				return TDS_FAIL;
 			if (tds->res_info->num_cols != 1)
 				break;
 			curcol = tds->res_info->columns[0];
@@ -559,7 +561,11 @@ tds_connect(TDSSOCKET * tds, TDSLOGIN * login, int *p_oserr)
 		if (tds->conn->spid == -1) {
 			strcat(str, "select @@spid ");
 		}
-		if (!db_selected && !tds_dstr_isempty(&login->database)) {
+		/* Select proper database if specified.
+		 * SQL Anywhere does not support multiple databases and USE statement
+		 * so don't send the request to avoid connection failures */
+		if (!db_selected && !tds_dstr_isempty(&login->database) &&
+		    (tds->conn->product_name == NULL || strcasecmp(tds->conn->product_name, "SQL Anywhere") != 0)) {
 			strcat(str, "use ");
 			tds_quote_id(tds, strchr(str, 0), tds_dstr_cstr(&login->database), -1);
 		}
@@ -668,7 +674,12 @@ tds_send_login(TDSSOCKET * tds, TDSLOGIN * login)
 
 	tds_put_login_string(tds, tds_dstr_cstr(&login->client_host_name), TDS_MAXNAME);	/* client host name */
 	tds_put_login_string(tds, tds_dstr_cstr(&login->user_name), TDS_MAXNAME);	/* account name */
-	tds_put_login_string(tds, tds_dstr_cstr(&login->password), TDS_MAXNAME);	/* account password */
+	/* account password */
+	if (login->encryption_level != TDS_ENCRYPTION_OFF) {
+		tds_put_login_string(tds, NULL, TDS_MAXNAME);
+	} else {
+		tds_put_login_string(tds, tds_dstr_cstr(&login->password), TDS_MAXNAME);
+	}
 	sprintf(blockstr, "%d", (int) getpid());
 	tds_put_login_string(tds, blockstr, TDS_MAXNAME);	/* host process */
 #ifdef WORDS_BIGENDIAN
@@ -680,7 +691,7 @@ tds_send_login(TDSSOCKET * tds, TDSLOGIN * login)
 #else
 	tds_put_n(tds, le1, 6);
 #endif
-	tds_put_byte(tds, login->bulk_copy);
+	tds_put_byte(tds, !login->bulk_copy);
 	tds_put_n(tds, NULL, 2);
 	if (IS_TDS42(tds->conn)) {
 		tds_put_int(tds, 512);
@@ -1204,6 +1215,10 @@ tds71_do_login(TDSSOCKET * tds, TDSLOGIN* login)
 	if (TDS_FAILED(ret))
 		return ret;
 
+	/* server just encrypt the first packet */
+	if (crypt_flag == 0)
+		tds->conn->encrypt_single_packet = 1;
+
 	ret = tds7_send_login(tds, login);
 
 	/* if flag is 0 it means that after login server continue not encrypted */
